
package com.tmerclub.cloud.platform.dto;

import io.swagger.v3.oas.annotations.media.Schema;
import jakarta.validation.constraints.NotNull;
import lombok.Data;

import java.io.Serial;
import java.io.Serializable;

/**
 * 网站配置信息表DTO
 *
 * @author Orange
 */
@Data
public class WebConfigDTO implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;
    @Schema(description = "主键")
    private Integer id;

    @Schema(description = "配置类型（1:平台端 2:商家端 3:PC端 4:h5端 6:供应商端）")
    @NotNull(message = "配置类型不能为空")
    private Integer configType;

    @Schema(description = "激活（0:否 1:是）")
    @NotNull(message = "激活状态不能为空")
    private Integer isActivity;

    @Schema(description = "登录logo")
    private String loginLogoImg;

    @Schema(description = "登录背景(后台+pc)")
    private String loginBgImg;

    @Schema(description = "版权声明-中文(后台+pc)")
    private String copyrightCn;

    @Schema(description = "版权声明-英文(后台+pc)")
    private String copyrightEn;

    @Schema(description = "网站标题文本-中文(后台+pc+h5)")
    private String titleContentCn;

    @Schema(description = "网站标题文本-英文(后台+pc+h5)")
    private String titleContentEn;

    @Schema(description = "标题图标(后台+pc)")
    private String titleImg;

    @Schema(description = "后台-菜单展开时的文本-中文")
    private String bsMenuTitleOpenCn;

    @Schema(description = "后台-菜单展开时的文本-英文")
    private String bsMenuTitleOpenEn;

    @Schema(description = "后台-菜单缩小时的文本-中文")
    private String bsMenuTitleCloseCn;

    @Schema(description = "后台-菜单缩小时的文本-英文")
    private String bsMenuTitleCloseEn;

    @Schema(description = "后台-顶部栏图标")
    private String bsTopBarIcon;

    @Schema(description = "PC-底部二维码")
    private String pcQrcodeImg;

    @Schema(description = "PC-公司全名-中文")
    private String pcCompanyNameCn;

    @Schema(description = "PC-公司全名-英文")
    private String pcCompanyNameEn;

    @Schema(description = "PC-公司名简写-中文")
    private String pcCompanyNameShortCn;

    @Schema(description = "PC-公司名简写-英文")
    private String pcCompanyNameShortEn;

    @Schema(description = "PC-公司图文logo（注册、登录、改密时显示）")
    private String pcLogoImgText;

    @Schema(description = "PC-导航栏欢迎语-中文")
    private String pcWelcomeCn;

    @Schema(description = "PC-导航栏欢迎语-英文")
    private String pcWelcomeEn;

    @Schema(description = "PC-底部公司地址等信息-中文")
    private String pcCompanyInfoCn;

    @Schema(description = "PC-底部公司地址等信息-英文")
    private String pcCompanyInfoEn;

}
