
package com.tmerclub.cloud.supplier.mapper;

import com.tmerclub.cloud.supplier.model.SupplierCompany;
import com.tmerclub.cloud.supplier.vo.SupplierCompanyVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 供应商工商信息
 *
 * @author lhd
 * @date 2022-03-14 17:04:32
 */
public interface SupplierCompanyMapper {
    /**
     * 根据供应商id获取供应商工商信息
     * @param supplierId
     * @param status
     * @return
     */
    SupplierCompanyVO getSupplierCompanyBySupplierIdAndStatus(@Param("supplierId") Long supplierId, @Param("status") Integer status);

    /**
     * 根据供应商工商信息id获取供应商工商信息
     *
     * @param supplierCompanyId 供应商工商信息id
     * @return 供应商工商信息
     */
    SupplierCompany getBySupplierCompanyId(@Param("supplierCompanyId") Long supplierCompanyId);

    /**
     * 保存供应商工商信息
     * @param supplierCompany 供应商工商信息
     */
    void save(@Param("supplierCompany") SupplierCompany supplierCompany);

    /**
     * 更新供应商工商信息
     * @param supplierCompany 供应商工商信息
     */
    void update(@Param("supplierCompany") SupplierCompany supplierCompany);

    /**
     * 根据供应商id更新供应商工商信息
     * @param supplierCompany
     * @param supplierId
     * @param temp 0代表通过审核，1代表平台修改
     */
    void updateBySupplierId(@Param("supplierCompany") SupplierCompany supplierCompany, @Param("supplierId") Long supplierId, @Param("temp") Integer temp);

    /**
     * 根据信用编码查询对应的供应商Id
     * @param creditCode
     * @return
     */
    List<Long> getSupplierIdByCreditCode(@Param("creditCode") String creditCode);

    /**
     * 根据供应商id更改供应商工商信息状态
     * @param supplierId
     * @param status
     * @param dbStatus
     */
    void batchUpdateStatus(@Param("supplierId") Long supplierId, @Param("status") Integer status, @Param("dbStatus") Integer dbStatus);

    /**
     * 根据供应商id和审核状态删除供应商工商信息
     * @param supplierId
     * @param status 审核状态
     * @return
     */
    void deleteBySupplierIdAndStatus(@Param("supplierId") Long supplierId, @Param("status") Integer status);

    /**
     * 查询供应商最近创建的工商信息 审核状态
     * @param supplierId
     * @return
     */
    Integer getLatestRecord(@Param("supplierId") Long supplierId);

    /**
     * 根据供应商id更新供应商工商信息
     * @param supplierCompany 供应商工商信息
     */
    void updateById(@Param("supplierCompany") SupplierCompany supplierCompany);

    /**
     * 更新影印件状态
     * @param supplierCompany
     */
    void updateIdCardById(@Param("supplierCompany") SupplierCompanyVO supplierCompany);

    /**
     * 更新正常的供应商工商信息为待审核
     */
    void updateStatusToWaitAudit();
}
