package com.tmerclub.cloud.warehouse.mapper;

import com.tmerclub.cloud.warehouse.model.Warehouse;
import com.tmerclub.cloud.warehouse.model.dto.WarehouseDTO;
import com.tmerclub.cloud.warehouse.model.vo.WarehouseVO;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;

/**
 * 仓库Mapper接口
 *
 * @author : frank
 * @create : 2025-04-14
 */
public interface WarehouseMapper {

    /**
     * 查询仓库列表
     *
     * @param warehouseDTO 查询参数
     * @return 仓库列表数据
     */
    List<WarehouseVO> list(@Param("dto") WarehouseDTO warehouseDTO);

    /**
     * 获取仓库详情
     *
     * @param id 主键
     * @return 仓库详情
     */
    WarehouseVO getById(Long id);

    /**
     * 新增仓库
     *
     * @param warehouse 新增参数
     * @return 新增结果
     */
    int save(Warehouse warehouse);

    /**
     * 修改仓库
     *
     * @param warehouse 修改参数
     * @return 修改结果
     */
    int update(Warehouse warehouse);
    /**
     * 根据经纬度获取最近的仓库信息
     *
     * @param lat 纬度
     * @param lng 经度
     * @return 最近的仓库信息
     */
    WarehouseVO getNearbyWarehouse(@Param("lat")BigDecimal lat, @Param("lng")BigDecimal lng);
}