package com.tmerclub.cloud.order.bo.mongo;

import com.tmerclub.cloud.order.constant.AllotOrderStatusEnum;
import lombok.Data;
import lombok.ToString;
import org.springframework.data.annotation.Id;
import org.springframework.data.mongodb.core.mapping.Document;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * @author gaozijie
 * @since 2023-11-24
 */
@Document("allot_order")
@Data
@ToString
public class MongoAllotOrderBO implements Serializable {

    @Serial
    private static final long serialVersionUID = 1L;

    /**
     * 调拨订单id
     */
    @Id
    private Long allotOrderId;

    /**
     * 调出点仓库id
     */
    private Long outWarehouseId;

    /**
     * 调出点库存点类型
     * @see com.tmerclub.cloud.common.product.constant.StockPointType
     */
    private Integer outStockPointType;

    /**
     * 调入点仓库id
     */
    private Long inWarehouseId;

    /**
     * 调入点库存点类型
     * @see com.tmerclub.cloud.common.product.constant.StockPointType
     */
    private Integer inStockPointType;

    /**
     * 物流公司id
     */
    private Long dvyCompanyId;

    /**
     * 物流方式
     * @see com.tmerclub.cloud.common.order.constant.DeliveryType
     */
    private Integer dvyType;

    /**
     * 物流单号
     */
    private String dvyOrderNumber;

    /**
     * 状态
     * @see AllotOrderStatusEnum
     */
    private Integer status;

    /**
     * 总调拨数量
     */
    private Integer totalAllotCount;

    /**
     * 备注
     */
    private String remark;

    /**
     * 店铺id
     */
    private Long shopId;

    /**
     * 系统类型
     * @see com.tmerclub.cloud.common.constant.SysTypeEnum
     */
    private Integer sysType;

    /**
     * 创建时间
     */
    private Date createTime;

    /**
     * 更新时间
     */
    private Date updateTime;

    /**
     * 调拨订单订单项
     */
    private List<MongoAllotOrderItemBO> allotOrderItemBOList;

}
