
package com.tmerclub.cloud.order.model;

import com.tmerclub.cloud.common.model.BaseModel;
import com.tmerclub.cloud.api.payment.constant.PaySysType;
import lombok.Data;
import lombok.ToString;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;

/**
 * 订单退款记录信息
 *
 * @author tmerclub
 * @date 2020-12-05 14:13:50
 */
@Data
@ToString
public class OrderRefund extends BaseModel implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;
    /**
     * 退款记录ID
     */
    private Long refundId;

    /**
     * 店铺ID
     */
    private Long shopId;
    /**
     * 供应商id
     */
    private Long supplierId;

    /**
     * 买家ID
     */
    private Long userId;

    /**
     * 订单号
     */
    private Long orderId;

    /**
     * 订单项ID(0:为全部订单项)
     */
    private Long orderItemId;

    /**
     * 支付系统类型
     *
     * @see PaySysType
     */
    private Integer paySysType;

    /**
     * 退货数量(0:为全部订单项)
     */
    private Integer refundCount;

    /**
     * 退还积分
     */
    private Long refundScore;

    /**
     * 退款金额
     */
    private Long refundAmount;

    /**
     * 平台退款金额（退款时将这部分钱退回给平台，所以商家要扣除从平台这里获取的金额）
     */
    private Long platformRefundAmount;

    /**
     * 平台退款佣金
     */
    private Long platformRefundCommission;

    /**
     * 退款单总分销金额
     */
    private Long distributionTotalAmount;

    /**
     * 退款单类型（1:整单退款,2:单个物品退款）
     */
    private Integer refundType;

    /**
     * 申请类型:1,仅退款,2退款退货
     */
    private Integer applyType;

    /**
     * 是否接收到商品(1:已收到,0:未收到)
     */
    private Integer isReceived;

    /**
     * 退款关闭原因(1.买家撤销退款 2.卖家拒绝退款 3.退款申请超时被系统关闭)
     */
    private Integer closeType;

    /**
     * 处理退款状态:(1.买家申请 2.卖家接受 3.买家发货 4.卖家收货 5.退款成功  -1.退款关闭)详情见ReturnMoneyStsType
     */
    private Integer returnMoneySts;

    /**
     * 申请原因 见 BuyerReasonType
     */
    private Integer buyerReason;

    /**
     * 申请说明
     */
    private String buyerDesc;

    /**
     * 联系方式（退款时留下的手机号码）
     */
    private String buyerMobile;

    /**
     * 文件凭证(逗号隔开)
     */
    private String imgUrls;

    /**
     * 商家/供应商文件凭证(逗号隔开)
     */
    private String shopImgUrls;

    /**
     * 超时时间（超过该时间不处理，系统将自动处理）（保留字段）
     */
    private Date overTime;

    /**
     * 拒绝原因
     */
    private String rejectMessage;

    /**
     * 卖家备注
     */
    private String sellerMsg;

    /**
     * 受理时间
     */
    private Date handelTime;

    /**
     * 发货时间
     */
    private Date deliveryTime;

    /**
     * 收货时间
     */
    private Date receiveTime;

    /**
     * 关闭时间
     */
    private Date closeTime;

    /**
     * 确定时间(确定退款时间)
     */
    private Date decisionTime;

    /**
     * 退款时间
     */
    private Date refundTime;

    /**
     * 移交供应商时间
     */
    private Date supplierTime;

    /**
     * 供应商处理状态， -1.商家自行处理 0.待供应商处理 1.供应商已同意 2.供应商已拒绝
     */
    private Integer supplierHandleStatus;
    /**
     * 采购退款金额
     */
    private Long purchaseRefundAmount;
    /**
     * 采购的平台佣金退款金额
     */
    private Long purPlatformRefundCommission;

    /**
     * 退款退回的赠品订单项ids
     */
    private String returnGiveawayIds;

    /**
     * 当前退款的实付金额（实付金额减去不退回赠品的金额）
     */
    private Long refundActualTotal;

    /**
     * 商家备注（商家将退款单移交供应商处理时保存）
     */
    private String shopSellerMsg;

    /**
     * 平台介入状态 -1.没有介入 1.用户申请介入 2.平台同意介入 3.平台拒绝介入 5.平台同意退款成功
     */
    private Integer platformInterventionStatus;

    /**
     * 用户申请介入理由
     */
    private String applyInterventionReason;

    /**
     * 用户申请介入凭证
     */
    private String applyInterventionImgUrls;

    /**
     * 平台介入退款方式 1.商家承担 2.商家供应商承担（供应商发货订单） 3.不同意退款
     */
    private Integer interventionRefundType;

    /**
     * 平台介入前的退款金额
     */
    private Long afterInterventionRefundAmount;

    /**
     * 平台留言
     */
    private String platformMessage;

    /**
     * 申请介入时间
     */
    private Date applyInterventionTime;

    /**
     * 介入完成时间
     */
    private Date interventionFinishTime;

}
