
package com.tmerclub.cloud.product.mapper;

import com.tmerclub.cloud.product.model.StockChangeReasonStatus;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 出入库原因状态
 *
 * @author lhd
 * @date 2022-06-22 16:57:27
 */
public interface StockChangeReasonStatusMapper {

    /**
     * 获取出入库原因状态列表
     * @return 出入库原因状态列表
     */
    List<StockChangeReasonStatus> list();

    /**
     * 根据出入库原因状态id获取出入库原因状态
     *
     * @param stockChangeReasonStatusId 出入库原因状态id
     * @return 出入库原因状态
     */
    StockChangeReasonStatus getByStockChangeReasonStatusId(@Param("stockChangeReasonStatusId") Long stockChangeReasonStatusId);

    /**
     * 保存出入库原因状态
     * @param stockChangeReasonStatus 出入库原因状态
     */
    void save(@Param("stockChangeReasonStatus") StockChangeReasonStatus stockChangeReasonStatus);

    /**
     * 更新出入库原因状态
     * @param stockChangeReasonStatus 出入库原因状态
     */
    void update(@Param("stockChangeReasonStatus") StockChangeReasonStatus stockChangeReasonStatus);

    /**
     * 根据出入库原因状态id删除出入库原因状态
     * @param stockChangeReasonStatusId
     */
    void deleteById(@Param("stockChangeReasonStatusId") Long stockChangeReasonStatusId);

    /**
     * 根据shopId 和 reasonId 获取(系统)出入库原因状态
     * @param shopId 店铺id
     * @param reasonId 出入库原因id
     */
    StockChangeReasonStatus getStatusByShopIdAndReasonId(@Param("shopId") Long shopId,
                                                         @Param("reasonId") Long reasonId);
}
