
package com.tmerclub.cloud.product.vo;

import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.tmerclub.cloud.common.model.ExcelModel;
import com.tmerclub.cloud.common.product.vo.BrandLangVO;
import com.tmerclub.cloud.common.product.vo.SpuLangVO;
import com.tmerclub.cloud.common.product.vo.SpuSkuAttrValueVO;
import lombok.Data;
import lombok.ToString;

import java.io.Serial;
import java.io.Serializable;
import java.util.List;

/**
 * @author xxw
 * @date 2022/6/30 9:47
 */
@Data
@ToString
public class SupplierSpuExcelVO extends ExcelModel implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;
    /**
     * excel 信息
     */
    public static final String EXCEL_NAME = "商品信息";
    public static final String SHEET_NAME = "商品";
    /**
     * 哪一行开始导出
     */
    public static final int MERGE_ROW_INDEX = 2;
    /**
     * 需要合并的列数组
     */
    public static final int[] MERGE_COLUMN_INDEX = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};

    @ExcelProperty(value = {"商品信息", "编号*"}, index = 0)
    private String seq;

    @ExcelProperty(value = {"商品信息", "商品中文名称*"}, index = 1)
    private String nameZh;


    @ExcelProperty(value = {"商品信息", "商品中文卖点"}, index = 3)
    private String sellingPointZh;


    @ExcelProperty(value = {"商品信息", "品牌"}, index = 5)
    private String brandName;

    @ExcelProperty(value = {"商品信息", "平台分类*"}, index = 6)
    private String categoryName;

    @ExcelProperty(value = {"商品信息", "店铺分类*"}, index = 7)
    private String shopCategoryName;

    @ExcelProperty(value = {"商品信息", "配送方式*"}, index = 8)
    private String deliveryMode;

    @ExcelProperty(value = {"商品信息", "运费模板"}, index = 9)
    private String deliveryTemplate;

    @ExcelProperty(value = {"商品信息", "固定运费金额(运费模板为固定运费时必填*)"}, index = 10)
    private String deliveryAmount;

    @ExcelProperty(value = {"商品信息", "状态*"}, index = 11)
    private String status;

    @ExcelProperty(value = {"商品信息", "所属供应商"}, index = 12)
    private String supplierName;

    @ExcelProperty(value = {"商品信息", "代销商品状态"}, index = 13)
    private String commissionSpuStatus;

    @ExcelProperty(value = {"商品信息", "发货方式"}, index = 14)
    private String supplierDelivery;

    @ExcelProperty({"商品sku信息", "中文sku名称"})
    private String skuNameZh;


    @ExcelProperty({"商品sku信息", "中文销售属性"})
    private String propertiesZh;


    @ExcelProperty({"商品sku信息", "售价"})
    private String priceFee;

    @ExcelProperty({"商品sku信息", "市场价"})
    private String marketPriceFee;

    @ExcelProperty({"商品sku信息", "供货价"})
    private String supplyPrice;

    @ExcelProperty({"商品sku信息", "库存"})
    private Integer stock;

    @ExcelProperty({"商品sku信息", "商品编码"})
    private String partyCode;

    @ExcelProperty({"商品sku信息", "商品重量"})
    private Double weight;

    @ExcelProperty({"商品sku信息", "商品体积"})
    private Double volume;

    /**
     * 商品id
     */
    @ExcelIgnore
    private Long spuId;

    /**
     * 物流模板id
     */
    @ExcelIgnore
    private Long deliveryTemplateId;

    @ExcelIgnore
    private List<SpuSkuAttrValueVO> spuSkuAttrValueList;

    @ExcelIgnore
    private List<BrandLangVO> brandLangList;

    @ExcelIgnore
    private List<SpuLangVO> spuLangList;

    @ExcelIgnore
    private Long categoryId;

    @ExcelIgnore
    private Long shopCategoryId;

    /**
     * 供应商id
     */
    @ExcelIgnore
    private Long supplierId;

    /**
     * 代销商品规格选择状态，1: 全选 2：部分选择
     */
    @ExcelIgnore
    private Integer skuChoiceStatus;

    /**
     * 发货方式 1.供应商发货 2.仓库发货
     */
    @ExcelIgnore
    private Integer supplierDeliveryType;

    /**
     * 供应商商品id
     */
    @ExcelIgnore
    private Long supplierSpuId;

}
